'use strict';

var react = require('react');
var reactDom = require('react-dom');

const canUsePip = typeof window !== "undefined" && !!window.documentPictureInPicture?.requestWindow;
const abortable = (asyncFn, signal, scheduleFn = queueMicrotask) => {
  return new Promise((resolve, reject) => {
    scheduleFn(() => {
      signal.aborted ? reject(signal.reason) : asyncFn().then(resolve, reject);
    });
  });
};
const PIPPortal = ({
  children,
  onClose,
  shouldCopyStyles = true,
  ...requestWindowOptions
}) => {
  const [container, setContainer] = react.useState(null);
  const setup = async () => {
    const pipWindow = await window.documentPictureInPicture.requestWindow(requestWindowOptions);
    setContainer(pipWindow.document.body);
    if (shouldCopyStyles) {
      copyStyles(pipWindow);
    }
    pipWindow.addEventListener("pagehide", () => {
      onClose?.();
    });
    return () => {
      pipWindow.close();
    };
  };
  react.useEffect(() => {
    const canUsePip2 = !!window.documentPictureInPicture?.requestWindow;
    if (!canUsePip2) {
      return;
    }
    const ac = new AbortController();
    const setupP = abortable(setup, ac.signal).catch((err) => {
      if (err !== "aborted") {
        console.error(err);
      }
    });
    return () => {
      ac.abort("aborted");
      setupP.then((x) => x?.());
    };
  }, []);
  return container ? reactDom.createPortal(children, container) : null;
};
const copyStyles = (pipWindow) => {
  [...document.styleSheets].forEach((styleSheet) => {
    try {
      const cssRules = [...styleSheet.cssRules].map((rule) => rule.cssText).join("");
      const style = document.createElement("style");
      style.textContent = cssRules;
      pipWindow.document.head.appendChild(style);
    } catch (e) {
      const link = document.createElement("link");
      link.rel = "stylesheet";
      link.type = styleSheet.type;
      link.media = styleSheet.media;
      link.href = styleSheet.href;
      pipWindow.document.head.appendChild(link);
    }
  });
};

exports.PIPPortal = PIPPortal;
exports.canUsePip = canUsePip;
